#!/usr/bin/env python3
#
# SPDX-FileCopyrightText: 2025 Carlos M. Fonseca <cmfonsec@dei.uc.pt>
#
# SPDX-License-Identifier: Apache-2.0

from api.neighborhood_constr_add import AddNeighbourhood
from api.neighborhood_local_2ex import TwoOptNeighbourhood
import math
import random
from api.solution import Solution

class Problem:
    def __init__(self, xy):
        self.n = len(xy)
        self.xy = tuple([(x,y) for x,y in xy])
        self.c_nbhood = None
        self.l_nbhood = None
        dist = []
        for i in range(self.n):
            aux = []
            for j in range(self.n):
                aux.append(math.sqrt((self.xy[i][0]-self.xy[j][0])**2 + (self.xy[i][1]-self.xy[j][1])**2))
            dist.append(tuple(aux))
        self.dist = tuple(dist)

    def __str__(self):
        out = [str(self.n)]
        for i, j in self.xy:
            out.append("%d %d" % (i, j))
        return "\n".join(out)

    def construction_neighbourhood(self):
        if self.c_nbhood is None:
            self.c_nbhood = AddNeighbourhood(self)
        return self.c_nbhood

    def local_neighbourhood(self):
        if self.l_nbhood is None:
            self.l_nbhood = TwoOptNeighbourhood(self)
        return self.l_nbhood

    @classmethod
    def from_textio(cls, f):
        """
        Create a problem from a text I/O source `f`
        """
        n = int(f.readline())
        xy = []
        for i in range(n):
            xy.append(tuple(map(float, f.readline().split())))
        return cls(xy)

    def empty_solution(self):
        return Solution(self, [0], set(range(1, self.n)), 0)

    def random_solution(self):
        c = list(range(1, self.n))
        random.shuffle(c)
        c.insert(0, 0)
        obj = self.dist[c[-1]][c[0]]
        for ix in range(1, self.n):
            obj += self.dist[c[ix-1]][c[ix]]
        return Solution(self, c, set(), obj)

