#!/usr/bin/env python3
#
# SPDX-FileCopyrightText: 2025 Carlos M. Fonseca <cmfonsec@dei.uc.pt>
#
# SPDX-License-Identifier: Apache-2.0

import roas.simple_solvers as solvers
import api.problem as tsp
import sys


if __name__ == '__main__':
    prob = tsp.Problem.from_textio(sys.stdin)
    sol = solvers.greedy_construction(prob.empty_solution())
    print("Greedy construction: {:.2f}".format(sol.objective_value()))
    sol1 = solvers.greedy_construction_random_tie_breaking(prob.empty_solution())
    print("Greedy construction with random tie breaking: {:.2f}".format(sol1.objective_value()))
    sol2 = prob.random_solution()
    print("Random solution: {:.2f}".format(sol2.objective_value()))
    print("Neighbourhood size:", len(list(prob.local_neighbourhood().moves(sol2))))
    sol3 = solvers.first_improvement(sol2.copy())
    print("First improvement: {:.2f}".format(sol3.objective_value()))
    sol4 = solvers.best_improvement(sol2.copy())
    print("Best improvement: {:.2f}".format(sol4.objective_value()))
    print("Starting at heuristic solution (gcwrtb)")
    sol5 = solvers.first_improvement(sol1.copy())
    print("First improvement: {:.2f}".format(sol5.objective_value()))
    sol6 = solvers.best_improvement(sol1.copy())
    print("Best improvement: {:.2f}".format(sol6.objective_value()))




